/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package roadsearch;

import java.awt.Color;
import java.util.Random;
import pathfinder.JPFMap;
import pathfinder.JPathFinder;

/** An example class that runs random path search (without any memory)
 *  instead of any requested search
 *
 * @author Petr Buryan
 */
public class RandomPathFinder extends JPathFinder{

    /** implements the random path search - searches a road from given start to 
     * target city but does not display the correct path (uses no memory)
     *@param start id of starting city
     *@param stop id of target city
     *@param map the map to be searched
     */
    public void performRandomSearch(int start, int stop, JPFMap map){
        //initialisation phase
        if (map==null)
            return;
        int node=start;
        Random rand=new Random();

        //search & visualisation phase
        while(node!=stop){
            try {
                Thread.sleep(100); //wait or a while so that the search can be seen in the visualisation (visualisation purposes)
            } catch (InterruptedException ex) {/**/}
            
            int[]neighbours=map.getCityNeighbours(node); //get ids of neighbouring cities
            int nextNeighbour=rand.nextInt(neighbours.length);
            int new_node=neighbours[nextNeighbour]; //go to a random neighbour
            
            inform("Going to: "+new_node); //show something to the user (text info)
            map.drawPath(node,new_node); //draw a line from the parent to the selected node (visualisation purposes)
            map.highlightCities(new int[]{new_node},Color.cyan);  //highlight the selected node (visualisation purposes)
            node=new_node; //switch parent to new node and repeat
            if(new_node==stop){ //target has been found => show something to the user and exit
                 map.drawPath(start,stop,Color.PINK); //show something to the user (visualisation purposes)
                 inform("Path discovered!"); //show something to the user (text info)
                 return; //exit
            }
        }
    }
    
    @Override
    public void performDFS(int start, int stop, JPFMap map) {
        performRandomSearch(start, stop, map);  //run the random search instead of DFS
    }

    @Override
    public void performBFS(int start, int stop, JPFMap map) {
        performRandomSearch(start, stop, map);  //run the random search instead of BFS
    }

    @Override
    public void searchForShortestPath(int start, int stop, JPFMap map) {
        performRandomSearch(start, stop, map);  //run the random search instead of A*
    }

    @Override
    public void searchForShortestTime(int start, int stop, JPFMap map) {
        performRandomSearch(start, stop, map);  //run the random search instead of A*
    }

}
